SystemOrganization addCategory: #'Bioloid-Core'!
SystemOrganization addCategory: #'Bioloid-Core-Protocol'!

Object subclass: #Bioloid
	instanceVariableNames: 'motors protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core'!

!Bioloid methodsFor: 'motors' stamp: 'Richo 9/19/2012 18:04'!
attach: aMotor
	aMotor isAttached
		ifTrue: [self error: ('{1} is already attached.' format: {aMotor}).
			^ self].
	aMotor setBioloid: self.
	motors add: aMotor! !

!Bioloid methodsFor: 'connection' stamp: 'sgb 10/24/2012 16:24'!
connectOnPort: portName
protocol connectOnPort: portName
! !

!Bioloid methodsFor: 'connection' stamp: 'Richo 9/19/2012 18:17'!
disconnect
	protocol disconnect! !

!Bioloid methodsFor: 'initialize-release' stamp: 'Richo 9/19/2012 18:05'!
initialize
	super initialize.
	protocol := BioloidProtocol new.
	motors := OrderedCollection new! !

!Bioloid methodsFor: 'connection' stamp: 'Richo 9/19/2012 18:17'!
isConnected
	^ protocol isConnected! !

!Bioloid methodsFor: 'accessing' stamp: 'Richo 9/19/2012 18:08'!
motors
	^ motors! !

!Bioloid methodsFor: 'accessing' stamp: 'Richo 9/19/2012 18:07'!
protocol
	^ protocol! !

Object subclass: #BioloidMotor
	instanceVariableNames: 'bioloid id led goalPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core'!

!BioloidMotor class methodsFor: 'instance creation' stamp: 'Richo 9/19/2012 17:59'!
id: motorId
	^ self basicNew initializeWith: motorId! !

!BioloidMotor class methodsFor: 'instance creation' stamp: 'sgb 10/24/2012 16:41'!
new
self shouldNotImplement ! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 15:54'!
goalPosition
	^ goalPosition ifNil: [0]
! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 15:52'!
goalPosition: aNumber
	self isAttached
	ifTrue:
	[goalPosition := aNumber.
	bioloid protocol
		writeOn: id
		address: 30
		word: aNumber]! !

!BioloidMotor methodsFor: 'initialize-release' stamp: 'Richo 9/19/2012 18:00'!
initializeWith: aNumber
	id := aNumber.
	self initialize! !

!BioloidMotor methodsFor: 'testing' stamp: 'Richo 9/19/2012 18:04'!
isAttached
	^ bioloid notNil! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 12:08'!
led 
	^ led ifNil: false! !

!BioloidMotor methodsFor: 'accessing' stamp: 'sgb 10/26/2012 12:01'!
led: aBoolean
	self isAttached
	ifTrue:
	[led:=aBoolean.
	bioloid protocol
		writeOn: id
		address: 25
		byte: aBoolean asBit]
! !

!BioloidMotor methodsFor: 'printing' stamp: 'Richo 9/19/2012 18:00'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(;
		print: id;
		nextPut: $)! !

!BioloidMotor methodsFor: 'private' stamp: 'Richo 9/19/2012 18:04'!
setBioloid: aBioloid 
	bioloid := aBioloid! !

Object subclass: #BioloidProtocol
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bioloid-Core-Protocol'!

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'Richo 9/19/2012 17:51'!
rqServoWriteByte
	^ 1! !

!BioloidProtocol class methodsFor: 'constants-request commands' stamp: 'Richo 9/19/2012 17:51'!
rqServoWriteWord
	^ 2! !

!BioloidProtocol methodsFor: 'converting' stamp: 'Richo 9/19/2012 17:48'!
asArgument: byte
	^ byte bitOr: 2r10000000! !

!BioloidProtocol methodsFor: 'converting' stamp: 'Richo 9/19/2012 17:48'!
asCommand: byte
	^ byte! !

!BioloidProtocol methodsFor: 'connection' stamp: 'Richo 9/19/2012 17:45'!
connectOnPort: aString
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	self isConnected ifTrue: [^ self].
	port := SerialPort new baudRate: 57600.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
! !

!BioloidProtocol methodsFor: 'connection' stamp: 'Richo 9/19/2012 17:45'!
disconnect
	self isConnected
		ifTrue: ["self stopSteppingProcess."
			port close.
			port := nil.
			self initialize]! !

!BioloidProtocol methodsFor: 'initialize-release' stamp: 'Richo 9/19/2012 17:45'!
initialize
	super initialize.
! !

!BioloidProtocol methodsFor: 'connection' stamp: 'Richo 9/19/2012 17:45'!
isConnected
	^ port notNil! !

!BioloidProtocol methodsFor: 'sending' stamp: 'Richo 9/19/2012 17:55'!
send: aByteArray
	Transcript show: aByteArray printString; cr.
	self isConnected ifFalse: [^ self].
	(port nextPutAll: aByteArray) < 0
		ifTrue: [
			"An error occurred, maybe the board was unplugged.
			Just in case, disconnect"
			self disconnect]! !

!BioloidProtocol methodsFor: 'commands' stamp: 'Richo 9/19/2012 17:55'!
writeOn: motorId address: address byte: value
	self send: {self asCommand: self class rqServoWriteByte.
				self asArgument: motorId.
				self asArgument: address.
				self asArgument: (value bitAnd: 2r01111111).
				self asArgument: (value bitAnd: 2r10000000) >> 7} asByteArray! !

!BioloidProtocol methodsFor: 'commands' stamp: 'Richo 9/19/2012 18:20'!
writeOn: motorId address: address word: value
	| byte1 byte2 |
	byte1 := value bitAnd: 2r11111111.
	byte2 := value >> 8.
	self send: {self asCommand: self class rqServoWriteWord.
				self asArgument: motorId.
				self asArgument: address.
				self asArgument: (byte1 bitAnd: 2r01111111).
				self asArgument: (byte1 bitAnd: 2r10000000) >> 7.
				self asArgument: (byte2 bitAnd: 2r01111111).
				self asArgument: (byte2 bitAnd: 2r10000000) >> 7} asByteArray! !
